﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace Workshops.Model.Validation
{
    public class ValidationManager : IValidationManager
    {
        private readonly Dictionary<Type, IList<IValidationRule>> _validationRuleMappings = new Dictionary<Type, IList<IValidationRule>>();

        public ValidationManager(IEnumerable<IValidationRule> rules)
        {
            foreach (var rule in rules)
            {
                // get entry in dictionary
                IList<IValidationRule> typeRulesList = null;
                if (_validationRuleMappings.ContainsKey(rule.TargetType))
                {
                    typeRulesList = _validationRuleMappings[rule.TargetType];
                }
                else
                {
                    typeRulesList = new List<IValidationRule>();
                    _validationRuleMappings.Add(rule.TargetType, typeRulesList);
                }

                // add rule
                typeRulesList.Add(rule);
            }
        }

        public ValidationResult Validate<T>(T businessObject) where T : IBusinessObject
        {
            // get type and rules
            var businessObjectType = businessObject.GetType();
            if (_validationRuleMappings.ContainsKey(businessObjectType))
            {
                var rules = _validationRuleMappings[businessObjectType];

                // apply rules
                foreach (var rule in rules)
                {
                    var validationResult = rule.Validate(businessObject);

                    if (validationResult.ResultType != ValidationResultType.Valid)
                    {
                        return validationResult;
                    }
                }
            }

            // no rules, no validation
            return ValidationResult.ValidResult;
        }
    }
}